package org.w3.directmapping

import org.w3.rdf.{Model => RDFModel, Implicits => RDFImplicits, _}
import org.w3.rdb.RDB._
import org.w3.sql
import org.w3.rdf.turtle.TurtleModule
import java.io.File

import org.scalatest.FunSuite

trait DirectMappingTest extends FunSuite with RDFModel with RDFImplicits with DirectMappingModule with TurtleModule {

  import DirectMapping._

  val SqlParser = sql.SqlParser()

  val turtleParser = new TurtleParser { }

  def testDirectMapping(testName:String, db:Database, expectedGraph:Graph):Unit =
    test(testName) {
      val computedGraph = databaseSemantics(db)
      assert (expectedGraph === computedGraph)
    }

  def testDirectMapping(testName:String, dbFile:File, expectedGraphFile:File):Unit = {
    val db = SqlParser.toDB(dbFile)
    val expectedGraph:Graph = turtleParser.toGraph(expectedGraphFile)
    testDirectMapping(testName, db, expectedGraph)
  }

  def testDirectMappingSpec(testName:String):Unit =
    testDirectMapping(testName,
		      new File("./sharedtestdata/directmappingspec/" + testName + ".sql"),
		      new File("./sharedtestdata/directmappingspec/" + testName + ".ttl"))

  def testDirectMapping(testName:String):Unit =
    testDirectMapping(testName,
		      new File("./sharedtestdata/tests/" + testName + ".sql"),
		      new File("./sharedtestdata/tests/" + testName + ".ttl"))

  // 2 People 1 Addresses
  testDirectMappingSpec("emp_addr")

  // 2 People 1 Addresses 1 Department
  testDirectMappingSpec("multi_key")

  // 2 People 1 Addresses 1 Department 2 Projects 1 Task
  testDirectMappingSpec("ref_no_pk")

  // 1 People 1 Addresses 1 Offices
  testDirectMappingSpec("hier_tabl_proto")

  // 1 People 1 Addresses 1 Offices 1 ExectutiveOffices
  testDirectMappingSpec("hier_tabl")

  // !!! goes in different module

  testDirectMapping("1table0rows")
  testDirectMapping("1table1row")
  testDirectMapping("1table2columns1row")
  testDirectMapping("1table3columns1row")
  testDirectMapping("varchar_varchar_1row")
  testDirectMapping("2duplicates0nulls")
}

