package org.w3.yourapp


object QueryManager {

  import java.net._
  import java.io._
  import java.util.jar._
  import scala.collection.JavaConversions._
  import scala.io.Source

  val clazz:Class[_] = this.getClass
  val classloader = this.getClass.getClassLoader

  /** http://www.uofr.net/~greg/java/get-resource-listing.html */
  def getResourceListing(path:String):List[String] = {
    var dirURL:URL = classloader.getResource(path)
    if (dirURL != null && dirURL.getProtocol == "file") {
      /* A file path: easy enough */
      new File(dirURL.toURI).list.toList
    } else {
      if (dirURL == null) {
        val me = clazz.getName().replace(".", "/")+".class"
        dirURL = classloader.getResource(me)
      }
      if (dirURL.getProtocol == "jar") {
        val jarPath = dirURL.getPath.substring(5, dirURL.getPath().indexOf("!"))
        val jar:JarFile = new JarFile(URLDecoder.decode(jarPath, "UTF-8"))
        val entries = jar.entries filter { _.getName startsWith path } map { e => {
          var entry = e.getName substring path.length
          val checkSubdir = entry indexOf "/"
          if (checkSubdir >= 0) entry = entry.substring(0, checkSubdir)
          entry
        } }
        
        entries
      }    
      sys.error("Cannot list files for URL "+dirURL);
    }
  }

  import scala.collection.mutable.LinkedHashMap

  def embeddedFiles(extension:String):LinkedHashMap[String, String] = {
    val pattern = extension.replaceAll("\\.", "\\.") + "$"
    val files = getResourceListing("queries/") filter { _.endsWith(extension) }
    val queries = files map { file => {
      val url = classloader.getResource("queries/" + file)
      val query = Source.fromURL(url, "UTF-8").getLines.mkString("\n")
      (file.replaceAll(pattern, ""), query)
    } }
    LinkedHashMap[String, String]() ++ queries
  }

  lazy val embeddedDDLs = embeddedFiles(".ddl")
  lazy val embeddedCONSTRUCTs = embeddedFiles(".construct")

}
