import sbt._
import Keys._


object Resolvers {
  val scalaToolsRepo = "Scala Tools Release Repository" at "http://scala-tools.org/repo-releases"
  val scalaToolsSnapRepo = "Scala Tools Snapshot Repository" at "http://scala-tools.org/repo-snapshots"
  val scalatestRepo = "org.scalatest" % "scalatest" % "1.2-for-scala-2.8.0.final-SNAPSHOT" % "test"
}

object Dependencies {
  val scalatest = "org.scalatest" % "scalatest_2.9.0" % "1.6.1"
  val liftweb = "net.liftweb" %% "lift-webkit" % "2.3" % "compile"
  val jetty = "org.eclipse.jetty" % "jetty-webapp" % "7.3.0.v20110203" % "jetty"
  val logback = "ch.qos.logback" % "logback-classic" % "0.9.26"
  val specs = "org.scala-tools.testing" % "specs_2.9.0-1" % "1.6.8" % "test"
  val dispatch = "net.databinder" %% "dispatch-http" % "0.8.4" % "test"
  val unfiltered_filter = "net.databinder" %% "unfiltered-filter" % "0.4.1"
  val unfiltered_jetty = "net.databinder" %% "unfiltered-jetty" % "0.4.1"
  val unfiltered_spec = "net.databinder" %% "unfiltered-spec" % "0.4.1" % "test"
  val unfiltered_scalate = "net.databinder" %% "unfiltered-scalate" % "0.4.1"
  val unfiltered_json = "net.databinder" %% "unfiltered-json" % "0.4.1"
  val slf4jSimple = "org.slf4j" % "slf4j-simple" % "1.6.1"
  val antiXML = "com.codecommit" %% "anti-xml" % "0.3-SNAPSHOT" % "test"
  val jena = "com.hp.hpl.jena" % "jena" % "2.6.4"
  val arq = "com.hp.hpl.jena" % "arq" % "2.8.8"
  val grizzled = "org.clapper" %% "grizzled-scala" % "1.0.7" % "test"
}

object BuildSettings {

  val buildOrganization = "org.w3"
  val buildVersion      = "0.1-SNAPSHOT"
  val buildScalaVersion = "2.9.0-1"

  val buildSettings = Defaults.defaultSettings ++ Seq (
    organization := buildOrganization,
    version      := buildVersion,
    scalaVersion := buildScalaVersion,
//    shellPrompt  := ShellPrompt.buildShellPrompt,
    parallelExecution in Test := true,
//    testFrameworks += TestFrameworks.Specs2,
    scalacOptions ++= Seq("-deprecation", "-unchecked") // ExplainTypes
  )
}

object FeDeRateBuild extends Build {

  import Resolvers._
  import Dependencies._
  import BuildSettings._
  import sbtassembly.Plugin._

  val ivyExclude =
    <dependencies>
      <dependency org="com.hp.hpl.jena" name="jena" rev="2.6.4">
        <exclude org="org.slf4j" module="slf4j-log4j12"/>
      </dependency>
      <dependency org="com.hp.hpl.jena" name="arq" rev="2.8.8">
        <exclude org="org.slf4j" module="slf4j-log4j12"/>
      </dependency>
    </dependencies> 

      // <dependency org="" name="" rev="">
      //   <exclude org="" module=""/>
      // </dependency>
      // <dependency org="" name="" rev="">
      //   <exclude org="" module=""/>
      // </dependency>

  val testSuiteSettings = Seq(
    libraryDependencies += scalatest
  )

  val testSettings = Seq(
    libraryDependencies += scalatest % "test"
  )

  val jenaSettings = Seq(
    ivyXML := ivyExclude
    // libraryDependencies += Dependencies.jena,
    // libraryDependencies += arq
  )

  val webappSettings = Seq(
//    libraryDependencies += specs,
    libraryDependencies += unfiltered_spec,
//    libraryDependencies += dispatch,
    libraryDependencies += unfiltered_filter,
    libraryDependencies += unfiltered_jetty,
    libraryDependencies += unfiltered_json,
    libraryDependencies += unfiltered_scalate,
    libraryDependencies += slf4jSimple
//      libraryDependencies += slf4jSimple,
  )

  lazy val federate = Project(
    id = "federate",
    base = file("."),
    settings = buildSettings ++ assemblySettings ++ Seq(
      jarName in Assembly := "directmapping.jar"
      ),
    aggregate = Seq(isomorphic,
                    rdb,
                    sql,
                    rdf,
                    rdfdsl,
                    turtle,
                    rdfmodel_test,
                    turtle,
                    turtle_test,
                    rdfxml,
                    rdfxml_test,
                    jena,
                    shared_test_data,
                    direct_mapping,
                    direct_mapping_test,
                    rdf2rdf,
                    direct_mapping_webapp)
  )

  lazy val isomorphic = Project(
    id = "isomorphic",
    base = file("isomorphic"),
    settings = buildSettings
  )

  lazy val rdb = Project(
    id = "rdb",
    base = file("rdb"),
    settings = buildSettings
  )

  lazy val sql = Project(
    id = "sql",
    base = file("sql"),
    settings = buildSettings ++ testSettings
  ) dependsOn (rdb)

  lazy val rdf = Project(
    id = "rdf",
    base = file("rdf"),
    settings = buildSettings
  ) dependsOn (isomorphic)

  lazy val rdfmodel_test = Project(
    id = "rdfmodel-test",
    base = file("rdfmodel-test"),
    settings = buildSettings ++ testSuiteSettings
  ) dependsOn (rdf)

  lazy val rdfdsl = Project(
    id = "rdfdsl",
    base = file("rdfdsl"),
    settings = buildSettings
  ) dependsOn (rdf)

  lazy val turtle = Project(
    id = "turtle",
    base = file("turtle"),
    settings = buildSettings
  ) dependsOn (rdf)

  lazy val turtle_test = Project(
    id = "turtle-test",
    base = file("turtle-test"),
    settings = buildSettings ++ testSuiteSettings
  ) dependsOn (turtle)

  lazy val rdfxml = Project(
    id = "rdfxml",
    base = file("rdfxml"),
    settings = buildSettings ++ jenaSettings
  ) dependsOn (rdf)

  lazy val rdfxml_test = Project(
    id = "rdfxml-test",
    base = file("rdfxml-test"),
    settings = buildSettings ++ jenaSettings ++ testSuiteSettings
  ) dependsOn (rdfxml)

  lazy val jena = Project(
    id = "jena",
    base = file("jena"),
    settings = buildSettings ++ jenaSettings
  ) dependsOn (rdf)

  lazy val shared_test_data = Project(
    id = "sharedtestdata",
    base = file("sharedtestdata"),
    settings = buildSettings ++ jenaSettings
  ) dependsOn (rdb, rdf, sql, turtle)
  
  
  lazy val direct_mapping = Project(
    id = "directmapping",
    base = file("directmapping"),
    settings = buildSettings
  ) dependsOn (rdb, rdf, sql, shared_test_data)
  
  lazy val direct_mapping_test = Project(
    id = "directmapping-test",
    base = file("directmapping-test"),
    settings = buildSettings ++ jenaSettings ++ testSuiteSettings
  ) dependsOn (direct_mapping, jena)

  lazy val rdf2rdf = Project(
    id = "rdf2rdf",
    base = file("rdf2rdf"),
    settings = buildSettings ++ jenaSettings ++ testSettings
  ) dependsOn (direct_mapping, jena)

  lazy val direct_mapping_webapp = Project(
    id = "directmapping-webapp",
    base = file("directmapping-webapp"),
    settings = buildSettings ++ jenaSettings ++ assemblySettings ++ webappSettings ++ Seq(
      jarName in Assembly := "directmapping.jar"
    )
  ) dependsOn (direct_mapping, jena, rdf2rdf)

//  lazy val jenaTest = project("jena-test", "jena-test", new JenaTest(_), jena, rdfmodelTest, directmappingTest, turtleTest, rdfxmlTest)
//  lazy val jena_test = Project(
//    id = "jena-test",
//    base = file("jena-test"),
//    settings = buildSettings ++ jenaSettings ++ testSettings
//  ) dependsOn (jena)





//  lazy val salatUtil = {
//    val id = "salat-util"
//    val base = file("salat-util")
//    val settings = buildSettings ++ Seq(
//      libraryDependencies ++= utilDeps,
//      libraryDependencies += slf4jSimple,
//      libraryDependencies <+= scalaVersion("org.scala-lang" % "scalap" % _)
//    )
//    Project(id = id, base = base, settings = settings)
//  }
//
//  lazy val salatCore = Project(
//    id = "salat-core",
//    base = file("salat-core"),
//    settings = buildSettings ++ Seq(libraryDependencies += casbah_core)
//  ) dependsOn (salatUtil)
//


}


//class FeDeRate(info: ProjectInfo) extends ParentProject(info) {
//  self =>
//
//  val feDeRatePath = Path.fromFile(".")
//
//  lazy val turtleTest = project("turtle-test", "turtle-test", new TurtleTest(_), turtle)
//  lazy val rdfxml = project("rdfxml", "rdfxml", new RDFXML(_), rdf)
//  lazy val rdfxmlTest = project("rdfxml-test", "rdfxml-test", new RDFXMLTest(_), rdfxml)
//  lazy val jena = project("jena", "jena", new Jena(_), rdf)
//  lazy val jenaTest = project("jena-test", "jena-test", new JenaTest(_), jena, rdfmodelTest, directmappingTest, turtleTest, rdfxmlTest)
//  lazy val rdf2rdf = project("rdf2rdf", "rdf2rdf", new RDF2RDF(_), jena, directmapping)
//  lazy val sharedtestdata = project("sharedtestdata", "sharedtestdata", new SharedTestData(_), rdb, rdf, sql, turtle)
//  lazy val directmapping = project("directmapping", "directmapping", new DirectMapping(_), rdb, rdf, sql, sharedtestdata)
//  lazy val directmappingWebapp = project("directmapping-webapp", "directmapping-webapp", new DirectMappingWebapp(_), directmapping, jena, rdf2rdf)
//  lazy val directmappingTest = project("directmapping-test", "directmapping-test", new DirectMappingTest(_), directmapping, jena)
//  lazy val sparql = project("sparql", "sparql", new SPARQL(_), rdf)
//  lazy val sparql2sql = project("sparql2sql", "sparql2sql", new SPARQL2SQL(_), sparql, sql)
//  lazy val sparql2sparql = project("sparql2sparql", "sparql2sparql", new SPARQL2SPARQL(_), sparql)
//  lazy val sparql2sparql2sql = project("sparql2sparql2sql", "sparql2sparql2sql", new SPARQL2SPARQL2SQL(_), sparql2sparql, sparql2sql)
//  lazy val sparql2sqlendpoint = project("sparql2sqlendpoint", "sparql2sqlendpoint", new SPARQL2SQLEndPoint(_), sparql2sql)
//  lazy val sparql2sqlendpointJar = project("sparql2sqlendpointjar", "sparql2sqlendpointjar", new SPARQL2SQLEndPointJar(_), sparql2sqlendpoint)
//
//  class Isomorphic(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class RDB(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class SQL(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class RDF(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class RDFDSL(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class RDFModelTest(info: ProjectInfo) extends DefaultProject(info) with TestSuite
//
//  class Turtle(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class TurtleTest(info: ProjectInfo) extends DefaultProject(info) with TestSuite
//
//  class RDFXML(info: ProjectInfo) extends DefaultProject(info) with Common {
//    val jena = "com.hp.hpl.jena" % "jena" % "2.6.4"
//    val jenaIri = "com.hp.hpl.jena" % "iri" % "0.8" from "http://openjena.org/repo/com/hp/hpl/jena/iri/0.8/iri-0.8.jar"
//  }
//
//  class RDFXMLTest(info: ProjectInfo) extends DefaultProject(info) with TestSuite
//
//  class Jena(info: ProjectInfo) extends DefaultProject(info) with Common {
//    val jena = "com.hp.hpl.jena" % "jena" % "2.6.4"
//    val jenaIri = "com.hp.hpl.jena" % "iri" % "0.8" from "http://openjena.org/repo/com/hp/hpl/jena/iri/0.8/iri-0.8.jar"
//  }
//
//  class JenaTest(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class RDF2RDF(info: ProjectInfo) extends DefaultProject(info) with Common {
////    val jena = "com.hp.hpl.jena" % "jena" % "2.6.4"
////    val jenaIri = "com.hp.hpl.jena" % "iri" % "0.8" from "http://openjena.org/repo/com/hp/hpl/jena/iri/0.8/iri-0.8.jar"
//    val arq = "com.hp.hpl.jena" % "arq" % "2.8.7"
//  }
//
//
//  class SharedTestData(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class DirectMapping(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class DirectMappingWebapp(info: ProjectInfo) extends DefaultWebProject(info) with Common {
//    val liftVersion = "2.3-RC5"
//    override def libraryDependencies = Set(
//      "net.liftweb" %% "lift-widgets" % liftVersion % "compile",
////     "net.liftweb" %% "lift-webkit" % liftVersion % "compile",
////     "net.liftweb" %% "lift-mapper" % liftVersion % "compile",
////     "com.h2database" % "h2" % "1.2.138",
//// 
//      "net.liftweb" %% "lift-webkit" % liftVersion % "compile->default",
//      // "org.mortbay.jetty" % "jetty" % "6.1.22" % "test->default",
//      "org.mortbay.jetty" % "jetty" % "6.1.22" % "compile,jetty",
//      // "org.eclipse.jetty" % "jetty-webapp" % "7.0.2.v20100331" % "compile,jetty",
//      // "org.eclipse.jetty" % "jetty-webapp" % "7.3.1.v20110307" % "compile,jetty",
//      "junit" % "junit" % "4.5" % "test->default",
//      "ch.qos.logback" % "logback-classic" % "0.9.26",
//      "org.scala-tools.testing" %% "specs" % "1.6.6" % "test->default"
//    ) ++ super.libraryDependencies
//
//    val jettyConf = config("jetty")
//    // val jettyDep = "org.eclipse.jetty" % "jetty-webapp" % "7.0.2.v20100331" % "compile,jetty"
//    override def jettyClasspath = managedClasspath(jettyConf)
//  }
//
//  class DirectMappingTest(info: ProjectInfo) extends DefaultProject(info) with TestSuite
//
//  class SPARQL(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class SPARQL2SQL(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class SPARQL2SPARQL(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class SPARQL2SQLEndPoint(info: ProjectInfo) extends DefaultWebProject(info) with Common with BSBMPlugin {
////    val jettyConf = config("jetty")
////    /* http://repo1.maven.org/maven2 */
////    val jettyDep = "org.eclipse.jetty" % "jetty-webapp" % "7.0.2.v20100331" % "compile,jetty"
////    override def jettyClasspath = managedClasspath(jettyConf)
////    val mysql = "mysql" % "mysql-connector-java" % "5.1.13"
////    val postgresql = "postgresql" % "postgresql" % "9.0-801.jdbc4"
////    val codaRepo = "Coda Hale's Repository" at "http://repo.codahale.com/"
////    val fig = "com.codahale" %% "fig" % "1.0.7" withSources()
////    override def webappUnmanaged = super.webappUnmanaged +++ ("src" / "main" / "resources" / "database.properties")
//  }
//
//  class SPARQL2SPARQL2SQL(info: ProjectInfo) extends DefaultProject(info) with Common
//
//  class SPARQL2SQLEndPointJar(info: ProjectInfo) extends DefaultProject(info) with ProguardProject {
//
//    // defines the default main class
//    override def mainClass: Option[String] = Some("org.w3.sparql2sqlendpoint.SPARQL2SQLEndpoint")
//
//    // tells Proguard to take some more JAR/WAR/.class
//    override def proguardInJars =
//      super.proguardInJars +++
//      // we need the Scala library
//      scalaLibraryPath +++
//      // the Proguard plugin does not considerer the .war files so we include the generated bytecode
//      (self.feDeRatePath / "sparql2sqlendpoint" / "target" / "scala_2.8.0" / "classes") +++
//      // ships the default configuration
//      (self.feDeRatePath / "sparql2sqlendpoint" / "src" / "main" / "resources")
//
//    override def proguardOptions = List(
//      proguardKeepAllScala,
//      // automatically generates the MANIFEST.MF
//      proguardKeepMain(mainClass.get),
//      // keeps all the SQL drivers we can find
//      "-keep public class * implements java.sql.Driver"
//    )
//
//  }
//
//}
