/* SparqlToSparqlToSql: convert SPARQL queries to sound SQL queries.
 * $Id$
 */

package org.w3.sparql2sparql2sql

import org.w3.rdb
import org.w3.sql
import org.w3.rdb.RDB.{RelName}
import org.w3.sparql
import org.w3.sparql2sparql
import org.w3.sparql2sql

object SparqlToSparqlToSql {

  def toView (rules:List[sparql.Construct], schema:rdb.RDB.Database, stemUri:sparql2sql.StemURI) = {

    /* For each rule... */
    val disjoints = rules.foldLeft(List[sql.Select]())((allgps, rule) => {

      /* ... turn each triple pattern in the rule heads into a SQL pattern. */
      rule.head.triplepatterns.foldLeft(allgps)((gps, triple) => {
	def substituteVar (term:sparql.Term, label:String, triple:sparql.TriplePattern):(sparql.Term, List[sparql.Var], List[sql.ProjectAttribute]) = {
	  term match {
	    case sparql.TermVar(v) => (sparql.TermVar(sparql.Var(label)), List(sparql.Var(label)), List[sql.ProjectAttribute]())
	    case sparql.TermUri(u) => (term, List[sparql.Var](), List(
	      sql.ProjectAttribute(sql.PrimaryExpressionTyped(rdb.RDB.Datatype.STRING,sql.Name(u.toString)),
				   sql.AttrAlias(sql.Name(label)))))
	    case sparql.TermBNode(b) => (term, List[sparql.Var](), List(
	      sql.ProjectAttribute(sql.PrimaryExpressionTyped(rdb.RDB.Datatype.STRING,sql.Name(b.toString + "(" + triple + ")")),
				   sql.AttrAlias(sql.Name(label)))))
	    case sparql.TermLit(l) => (term, List[sparql.Var](), List(
	      sql.ProjectAttribute(sql.PrimaryExpressionTyped(rdb.RDB.Datatype.STRING,sql.Name(l.toString)),
				   sql.AttrAlias(sql.Name(label)))))
	  }
	}

	/* Add constant for predicate into selects. */

	/* Substituted SPARQL ?S and ?O into the triple pattern. */
	val (s, sselects, sconsts) = substituteVar(triple.s, "S", triple);
	val (p, pselects, pconsts) = substituteVar(triple.p, "P", triple);
	val (o, oselects, oconsts) = substituteVar(triple.o, "O", triple);
	val select = sparql.Select(
	  false,
	  sparql.SparqlAttributeList(sselects ++ pselects ++ oselects),
	  sparql.TriplesBlock(List(sparql.TriplePattern(s, p, o))),
	  List[sparql.OrderElt](), None, None
	)

	/* Convert to equivalent SQL SELECT. */
	val emptyPatternMap = sparql2sparql.NodePatternMap(Map[sparql.Var, sparql2sparql.NodePattern]())
	val asStem = sparql2sparql.SparqlToSparql(select, List(sparql2sparql.SparqlMap(rule, emptyPatternMap)))
	// println("triple: "+triple)
	// println("asStem: "+asStem)
	val sql.Select(distinct, sql.Projection(attributes), tablelist, expression, order,  offset, limit) = sparql2sql.SparqlToSql(schema, asStem, stemUri, false, true)._1
	// println("SQL: "+sql.Select(sql.Projection(attributes ++ sconsts ++ pconsts ++ oconsts), tablelist, expression))

	/* Add resulting SELECT to list of graph patterns. */
	gps :+ sql.Select(distinct, sql.Projection(attributes ++ sconsts ++ pconsts ++ oconsts), tablelist, expression, order, offset, limit)
      })
    })

    /* Create a view from the disjoints.*/
    val body:sql.SelectORUnion = disjoints.size match {
      case 0 => error("not sure why you want a view with no rows...")
      case 1 => disjoints.toList(0)
      case _ => sql.Union(disjoints.toSet)
    }
    sql.View(rdb.RDB.RelName("triples"), body)
  }
}

