package org.w3.rdf.turtle

import org.w3.rdf.{Model => RDFModel, Implicits => RDFImplicits, _}
import org.scalatest.FunSuite

trait TurtleTest extends FunSuite with TurtleModule with RDFImplicits {

  val TurtleParser = new TurtleParser {  }

  test("directgraph_emp_adder") {
    val directgraph_emp_adder:Graph =
      Graph(
	Triple(IRI("People/ID.7#_"),IRI("People#ID"),TypedLiteral("7",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.7#_"),IRI("People#fname"),TypedLiteral("Bob",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#addr"),IRI("Addresses/ID.18#_")),
	Triple(IRI("People/ID.8#_"),IRI("People#ID"),TypedLiteral("8",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.8#_"),IRI("People#fname"),TypedLiteral("Sue",IRI("http://www.w3.org/2001/XMLSchema#string"))), 
	
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#city"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#state"),TypedLiteral("MA",IRI("http://www.w3.org/2001/XMLSchema#string")))
      )

    val turtle_emp_adder:String = """
@prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
<People/ID.7#_> <People#ID> "7"^^<http://www.w3.org/2001/XMLSchema#integer> .
<People/ID.7#_> <People#fname> "Bob"^^<http://www.w3.org/2001/XMLSchema#string> .
<People/ID.7#_> <People#addr> <Addresses/ID.18#_> .
<People/ID.8#_> <People#ID> 8 .
<People/ID.8#_> <People#fname> "Sue"^^xsd:string .
<Addresses/ID.18#_> <Addresses#ID> 18 .
<Addresses/ID.18#_> <Addresses#city> "Cambridge"^^xsd:string .
<Addresses/ID.18#_> <Addresses#state> "MA"^^xsd:string .
"""
    assert(directgraph_emp_adder === TurtleParser.toGraph(turtle_emp_adder))
  }

  test("directgraph_multi_key") {
    val directgraph_multi_key:Graph =
      Graph(
	Triple(IRI("People/ID.7#_"),IRI("People#deptName"),TypedLiteral("accounting",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#deptCity"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#deptName_deptCity"),IRI("Department/ID.23#_")),

	Triple(IRI("Department/ID.23#_"),IRI("Department#ID"),TypedLiteral("23",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Department/ID.23#_"),IRI("Department#name"),TypedLiteral("accounting",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Department/ID.23#_"),IRI("Department#city"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Department/ID.23#_"),IRI("Department#manager"),IRI("People/ID.8#_")),

	Triple(IRI("People/ID.7#_"),IRI("People#ID"),TypedLiteral("7",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.7#_"),IRI("People#fname"),TypedLiteral("Bob",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#addr"),IRI("Addresses/ID.18#_")),
	Triple(IRI("People/ID.8#_"),IRI("People#ID"),TypedLiteral("8",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.8#_"),IRI("People#fname"),TypedLiteral("Sue",IRI("http://www.w3.org/2001/XMLSchema#string"))), 

	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#city"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#state"),TypedLiteral("MA",IRI("http://www.w3.org/2001/XMLSchema#string"))))

    val turtle_multi_key:String = """
@prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
<People/ID.7#_> <People#deptName> "accounting"^^xsd:string .
<People/ID.7#_> <People#deptCity> "Cambridge"^^xsd:string .
<People/ID.7#_> <People#deptName_deptCity> <Department/ID.23#_> .

<Department/ID.23#_> <Department#ID> "23"^^xsd:integer .
<Department/ID.23#_> <Department#name> "accounting"^^xsd:string .
<Department/ID.23#_> <Department#city> "Cambridge"^^xsd:string .
<Department/ID.23#_> <Department#manager> <People/ID.8#_> .

<People/ID.7#_> <People#ID> "7"^^xsd:integer .
<People/ID.7#_> <People#fname> "Bob"^^xsd:string .
<People/ID.7#_> <People#addr> <Addresses/ID.18#_> .
<People/ID.8#_> <People#ID> "8"^^xsd:integer .
<People/ID.8#_> <People#fname> "Sue"^^xsd:string . 

<Addresses/ID.18#_> <Addresses#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <Addresses#city> "Cambridge"^^xsd:string .
<Addresses/ID.18#_> <Addresses#state> "MA"^^xsd:string .
"""
    assert(directgraph_multi_key === TurtleParser.toGraph(turtle_multi_key))
  }

  test("directgraph_ref_no_pk") {
    val directgraph_ref_no_pk:Graph =
      Graph(
	Triple(BNode("a"), IRI("Projects#lead"), IRI("People/ID.8#_")),
	Triple(BNode("a"), IRI("Projects#name"), TypedLiteral("pencil survey", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(BNode("a"), IRI("Projects#deptName"), TypedLiteral("accounting", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(BNode("a"), IRI("Projects#deptCity"), TypedLiteral("Cambridge", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(BNode("a"), IRI("Projects#deptName_deptCity"), IRI("Department/ID.23#_")),

	Triple(BNode("b"), IRI("Projects#lead"), IRI("People/ID.8#_")),
	Triple(BNode("b"), IRI("Projects#name"), TypedLiteral("eraser survey", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(BNode("b"), IRI("Projects#deptName"), TypedLiteral("accounting", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(BNode("b"), IRI("Projects#deptCity"), TypedLiteral("Cambridge", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(BNode("b"), IRI("Projects#deptName_deptCity"), IRI("Department/ID.23#_")),

	Triple(IRI("TaskAssignments/worker.7_project.pencil+survey#_"), IRI("TaskAssignments#worker"), IRI("People/ID.7#_")),
	Triple(IRI("TaskAssignments/worker.7_project.pencil+survey#_"), IRI("TaskAssignments#project"), TypedLiteral("pencil survey", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("TaskAssignments/worker.7_project.pencil+survey#_"), IRI("TaskAssignments#deptName"), TypedLiteral("accounting", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("TaskAssignments/worker.7_project.pencil+survey#_"), IRI("TaskAssignments#deptCity"), TypedLiteral("Cambridge", IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("TaskAssignments/worker.7_project.pencil+survey#_"), IRI("TaskAssignments#deptName_deptCity"), IRI("Department/ID.23#_")),
	Triple(IRI("TaskAssignments/worker.7_project.pencil+survey#_"), IRI("TaskAssignments#project_deptName_deptCity"), BNode("a")),

	Triple(IRI("People/ID.7#_"),IRI("People#deptName"),TypedLiteral("accounting",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#deptCity"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#deptName_deptCity"),IRI("Department/ID.23#_")),

	Triple(IRI("Department/ID.23#_"),IRI("Department#ID"),TypedLiteral("23",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Department/ID.23#_"),IRI("Department#name"),TypedLiteral("accounting",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Department/ID.23#_"),IRI("Department#city"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Department/ID.23#_"),IRI("Department#manager"),IRI("People/ID.8#_")),

	Triple(IRI("People/ID.7#_"),IRI("People#ID"),TypedLiteral("7",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.7#_"),IRI("People#fname"),TypedLiteral("Bob",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#addr"),IRI("Addresses/ID.18#_")),
	Triple(IRI("People/ID.8#_"),IRI("People#ID"),TypedLiteral("8",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.8#_"),IRI("People#fname"),TypedLiteral("Sue",IRI("http://www.w3.org/2001/XMLSchema#string"))), 

	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#city"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#state"),TypedLiteral("MA",IRI("http://www.w3.org/2001/XMLSchema#string")))
      )

    val turtle_ref_no_pk:String = """
@prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
_:a <Projects#lead> <People/ID.8#_> .
_:a <Projects#name> "pencil survey"^^xsd:string .
_:a <Projects#deptName> "accounting"^^xsd:string .
_:a <Projects#deptCity> "Cambridge"^^xsd:string .
_:a <Projects#deptName_deptCity> <Department/ID.23#_> .

_:b <Projects#lead> <People/ID.8#_> .
_:b <Projects#name> "eraser survey"^^xsd:string .
_:b <Projects#deptName> "accounting"^^xsd:string .
_:b <Projects#deptCity> "Cambridge"^^xsd:string .
_:b <Projects#deptName_deptCity> <Department/ID.23#_> .

<TaskAssignments/worker.7_project.pencil+survey#_> <TaskAssignments#worker> <People/ID.7#_> .
<TaskAssignments/worker.7_project.pencil+survey#_> <TaskAssignments#project> "pencil survey"^^xsd:string .
<TaskAssignments/worker.7_project.pencil+survey#_> <TaskAssignments#deptName> "accounting"^^xsd:string .
<TaskAssignments/worker.7_project.pencil+survey#_> <TaskAssignments#deptCity> "Cambridge"^^xsd:string .
<TaskAssignments/worker.7_project.pencil+survey#_> <TaskAssignments#deptName_deptCity> <Department/ID.23#_> .
<TaskAssignments/worker.7_project.pencil+survey#_> <TaskAssignments#project_deptName_deptCity> _:a .

<People/ID.7#_> <People#deptName> "accounting"^^xsd:string .
<People/ID.7#_> <People#deptCity> "Cambridge"^^xsd:string .
<People/ID.7#_> <People#deptName_deptCity> <Department/ID.23#_> .

<Department/ID.23#_> <Department#ID> "23"^^xsd:integer .
<Department/ID.23#_> <Department#name> "accounting"^^xsd:string .
<Department/ID.23#_> <Department#city> "Cambridge"^^xsd:string .
<Department/ID.23#_> <Department#manager> <People/ID.8#_> .

<People/ID.7#_> <People#ID> "7"^^xsd:integer .
<People/ID.7#_> <People#fname> "Bob"^^xsd:string .
<People/ID.7#_> <People#addr> <Addresses/ID.18#_> .
<People/ID.8#_> <People#ID> "8"^^xsd:integer .
<People/ID.8#_> <People#fname> "Sue"^^xsd:string .

<Addresses/ID.18#_> <Addresses#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <Addresses#city> "Cambridge"^^xsd:string .
<Addresses/ID.18#_> <Addresses#state> "MA"^^xsd:string .
"""
    assert(directgraph_ref_no_pk === TurtleParser.toGraph(turtle_ref_no_pk))
  }

  test("directgraph_hier_tabl_proto") {
    val directgraph_hier_tabl_proto:Graph =
      Graph(
	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#building"),TypedLiteral("32",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#ofcNumber"),TypedLiteral("G528",IRI("http://www.w3.org/2001/XMLSchema#string"))), 

	Triple(IRI("People/ID.7#_"),IRI("People#ID"),TypedLiteral("7",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.7#_"),IRI("People#fname"),TypedLiteral("Bob",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#addr"),IRI("Addresses/ID.18#_")),

	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#city"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#state"),TypedLiteral("MA",IRI("http://www.w3.org/2001/XMLSchema#string")))
      )
    val turtle_hier_tabl_proto:String = """
@prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
<Addresses/ID.18#_> <Offices#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <Offices#building> "32"^^xsd:integer .
<Addresses/ID.18#_> <Offices#ofcNumber> "G528"^^xsd:string . 

<People/ID.7#_> <People#ID> "7"^^xsd:integer .
<People/ID.7#_> <People#fname> "Bob"^^xsd:string .
<People/ID.7#_> <People#addr> <Addresses/ID.18#_> .

<Addresses/ID.18#_> <Addresses#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <Addresses#city> "Cambridge"^^xsd:string .
<Addresses/ID.18#_> <Addresses#state> "MA"^^xsd:string .
"""
    assert(directgraph_hier_tabl_proto === TurtleParser.toGraph(turtle_hier_tabl_proto))
  }

  test("directgraph_hier_tabl") {
    val directgraph_hier_tabl:Graph =
      Graph(
	Triple(IRI("Addresses/ID.18#_"),IRI("ExecutiveOffices#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("ExecutiveOffices#desk"),TypedLiteral("oak",IRI("http://www.w3.org/2001/XMLSchema#string"))), 

	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#building"),TypedLiteral("32",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#ofcNumber"),TypedLiteral("G528",IRI("http://www.w3.org/2001/XMLSchema#string"))), 

	Triple(IRI("People/ID.7#_"),IRI("People#ID"),TypedLiteral("7",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.7#_"),IRI("People#fname"),TypedLiteral("Bob",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#addr"),IRI("Addresses/ID.18#_")),

	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#city"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#state"),TypedLiteral("MA",IRI("http://www.w3.org/2001/XMLSchema#string")))
      )

    val turtle_hier_tabl:String = """
@prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
<Addresses/ID.18#_> <ExecutiveOffices#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <ExecutiveOffices#desk> "oak"^^xsd:string . 

<Addresses/ID.18#_> <Offices#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <Offices#building> "32"^^xsd:integer .
<Addresses/ID.18#_> <Offices#ofcNumber> "G528"^^xsd:string . 

<People/ID.7#_> <People#ID> "7"^^xsd:integer .
<People/ID.7#_> <People#fname> "Bob"^^xsd:string .
<People/ID.7#_> <People#addr> <Addresses/ID.18#_> .

<Addresses/ID.18#_> <Addresses#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <Addresses#city> "Cambridge"^^xsd:string .
<Addresses/ID.18#_> <Addresses#state> "MA"^^xsd:string .
"""
    assert(directgraph_hier_tabl === TurtleParser.toGraph(turtle_hier_tabl))
  }

}

class TurtleSerializerTest () extends FunSuite with TurtleModule with ConcreteModel with RDFImplicits {

  val TurtleParser = new TurtleParser {  }

  val Ser = new TurtleSerializer()

  test("directgraph_serializer1") {
    val turtle_serializer1:String = """
@prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
<Addresses/ID.18#_> <ExecutiveOffices#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <ExecutiveOffices#desk> "oak"^^xsd:string . 

<Addresses/ID.18#_> <Offices#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <Offices#building> "32"^^xsd:integer .
<Addresses/ID.18#_> <Offices#ofcNumber> "G528"^^xsd:string . 

<People/ID.7#_> <People#ID> "7"^^xsd:integer .
<People/ID.7#_> <People#fname> "Bob"^^xsd:string .
<People/ID.7#_> <People#addr> <Addresses/ID.18#_> .

<Addresses/ID.18#_> <Addresses#ID> "18"^^xsd:integer .
<Addresses/ID.18#_> <Addresses#city> "Cambridge"^^xsd:string .
<Addresses/ID.18#_> <Addresses#state> "MA"^^xsd:string .
"""
    val directgraph_serializer1:Graph =
      Graph(
	Triple(IRI("Addresses/ID.18#_"),IRI("ExecutiveOffices#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("ExecutiveOffices#desk"),TypedLiteral("oak",IRI("http://www.w3.org/2001/XMLSchema#string"))), 

	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#building"),TypedLiteral("32",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Offices#ofcNumber"),TypedLiteral("G528",IRI("http://www.w3.org/2001/XMLSchema#string"))), 

	Triple(IRI("People/ID.7#_"),IRI("People#ID"),TypedLiteral("7",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("People/ID.7#_"),IRI("People#fname"),TypedLiteral("Bob",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("People/ID.7#_"),IRI("People#addr"),IRI("Addresses/ID.18#_")),

	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#ID"),TypedLiteral("18",IRI("http://www.w3.org/2001/XMLSchema#integer"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#city"),TypedLiteral("Cambridge",IRI("http://www.w3.org/2001/XMLSchema#string"))),
	Triple(IRI("Addresses/ID.18#_"),IRI("Addresses#state"),TypedLiteral("MA",IRI("http://www.w3.org/2001/XMLSchema#string")))
      )
    assert(Ser.graphString(directgraph_serializer1) === turtle_serializer1)
  }

}

